from dataclasses import dataclass
import requests

@dataclass
class PyDawa:
    
    srid: str = '25832'
    format: str = 'json'
    struktur: str = 'mini'

    def get_koordinater(self, json_response):
        """Henter koordinater for en given adresse.

        :param json_response: Response fra dawa api. Skal være i json format.

        :returns: En tuple med koordinater.
        :rtype: tuple
        """
        if self.struktur == 'mini':
            return (json_response['x'], json_response['y'])
        elif self.struktur == 'flad':
            return (json_response['vejpunkt_x'], json_response['vejpunkt_y'])
        elif self.struktur == 'nestet':
            return (json_response['adgangsadresse']['adgangspunkt']['koordinater'][0], json_response['adgangsadresse']['adgangspunkt']['koordinater'][1])

@dataclass
class Adressesoeg(PyDawa):
    """Søg efter en adresse i DAR.

        :param q: Adresse, som skal søges efter.
        :param struktur: Strukturen af returneret data fra API. Mulige værdier: "nestet", "flad" eller "mini". Det anbefales at benytte mini-formatet hvis der ikke er behov for den fulde struktur, da dette vil give bedre svartider (DEFAULT mini)
        :param srid: SRID for det koordinatsystem koordinaterne er i (DEFAULT 25832).
        :param format: Mulige værdier json (DEFAULT), geojson (hvor det giver mening), jsonp, ndjson, csv.

        :returns: En liste med svar fra API.
        :rtype: list
    """
    q: str = None
    vejnavn: str = ''
    husnr: str = ''
    postnr: str = ''
    
    def info(self):
        """Henter information om adressen fra DAWA API.
        """
        if self.q == None:
            params = {'vejnavn': self.vejnavn, 'husnr': self.husnr, 'postnr': self.postnr, 'struktur': self.struktur, 'format': self.format, 'srid': self.srid}
            url = 'http://dawa.aws.dk/adgangsadresser'
            response = requests.get(url, params=params)
            if self.format == 'json' or self.format == 'geojson':
                response = response.json()
            else:
                response = response.text()
            return response
        else:
            params = {'q': self.q, 'struktur': self.struktur, 'format': self.format, 'srid': self.srid}
            url = 'http://dawa.aws.dk/adgangsadresser'
            response = requests.get(url, params=params)
            if self.format == 'json' or self.format == 'geojson':
                response = response.json()
            else:
                response = response.text()
            return response

@dataclass
class Adresseopslag(PyDawa):
    """Henter en adresse fra DAR udfra en adresse id.

    :param id: Adressens unikke id.
    :param struktur: Strukturen af returneret data fra API. Mulige værdier: "nestet", "flad" eller "mini". Det anbefales at benytte mini-formatet hvis der ikke er behov for den fulde struktur, da dette vil give bedre svartider (DEFAULT mini)
    :param srid: SRID for det koordinatsystem koordinaterne er i (DEFAULT 25832).
    :param format: Mulige værdier json (DEFAULT), geojson (hvor det giver mening), jsonp, ndjson, csv.

    :returns: En liste med svar fra API.
    :rtype: list
    """
    id: str = None

    def info(self):
        """Henter information om adressen fra DAWA API.
        """
        params = {'struktur': self.struktur, 'srid': self.srid, 'format': self.format}
        url = 'http://dawa.aws.dk/adgangsadresser/' + self.id 
        response = requests.get(url, params=params)
        if self.format == 'json' or self.format == 'geojson':
            response = response.json()
        else:
            response = response.text()
        return response

@dataclass
class Adressevasker(PyDawa):
    """Vasker en adresse og kommer med bedste gæt på en rigtig adresse. 

    :param betegnelse: Adresse, som skal vaskes

    :returns: En eller flere adresser, som bedst matcher svaret
    :rtype: dictionary
    """
    betegnelse: str = None

    def info(self):
        params = {'betegnelse': self.betegnelse}
        url = 'http://dawa.aws.dk/datavask/adgangsadresser'
        response = requests.get(url, params=params)
        return response.json()

@dataclass
class Reverse(PyDawa):
    """Returnere den nærmeste adresse til en given koordinat.

    :param koordinater: En tuple med koordinater.

    :returns: Den nærmeste adresse til et givent koordinat.
    """
    
    koordinater: tuple = ()

    def info(self):
        params = {'x': self.koordinater[0], 'y': self.koordinater[1], 'struktur': self.struktur, 'format': self.format}
        url = 'http://dawa.aws.dk/adgangsadresser/reverse'
        response = requests.get(url, params=params)
        return response.json()


import pandas as pd
from pathlib import Path
from dataclasses import dataclass
from .dawa import *
import csv

@dataclass
class Geokoder:
    """En Class, som tager i mod en sti til en csv eller en xlsx fil, og kan geokode den.

    Parameters:
        filepath (str): Placeringen af filen.
    """
    filepath: str
    cols: None = None

    def __post_init__(self):
        self.filepath = Path(self.filepath)
        if not self.filepath.exists():
            raise FileNotFoundError(f'{self.filepath} findes ikke!')
        self.extension = self.filepath.suffix

    def progbar(self, curr, total, full_progbar):
        frac = curr / total
        filled_progbar = round(frac * full_progbar)
        print('\r', '#' * filled_progbar + '-' * (full_progbar - filled_progbar), f'[{frac:>7.2%}]', end='')

    def geokod(self, save=False):

        def geocode_df(dataframe):
            total_len = len(dataframe.index)
            for index, row in dataframe.iterrows():
                search_string = ' '.join([str(row[col]) for col in self.cols])
                adresse = Adressesoeg(q=search_string)
                adresse_data = adresse.info()
                if len(adresse_data) > 0:
                    koordinat = adresse.get_koordinater(adresse_data[0])
                    dataframe.set_value(index, 'x', koordinat[0])
                    dataframe.set_value(index, 'y', koordinat[1])
                else:
                    adresse_vask = Adressevasker(betegnelse=search_string)
                    response_vask = adresse_vask.info()
                    adr_id = response_vask['resultater'][0]['adresse']['id']
                    rigtig_adresse = Adresseopslag(id=adr_id)
                    rigtig_adresse_data = rigtig_adresse.info()
                    try:
                        koordinat = rigtig_adresse.get_koordinater(rigtig_adresse_data)
                        dataframe.set_value(index, 'x', koordinat[0])
                        dataframe.set_value(index, 'y', koordinat[1])
                    except:
                        dataframe.set_value(index, 'x', 'NaN')
                        dataframe.set_value(index, 'y', 'NaN')
                self.progbar(index + 1, total_len, 100)
            return dataframe

        if self.extension == '.csv':
            sep = self.find_csv_sep()
            df = pd.read_csv(self.filepath, sep=sep)
            df_geokod = geocode_df(df)
            if save == True:
                df_geokod.to_csv(str(self.filepath.parent.joinpath(f'{self.filepath.stem}_geokodet{self.filepath.suffix}')))
            else:
                return df_geokod

        elif self.extension =='.xlsx':
            df = pd.read_excel(self.filepath)
            df_geokod = geocode_df(df)
            if save == True:
                df_geokod.to_excel(str(self.filepath.parent.joinpath(f'{self.filepath.stem}_geokodet{self.filepath.suffix}')))
            else:
                return df_geokod

    def find_csv_sep(self):
        sniffer = csv.Sniffer()
        with open(self.filepath, 'r') as f:
            line = f.readline()
            dialect = sniffer.sniff(line)
        return dialect.delimiter


import pytest
import pandas as pd
from pydawa import Geokoder
from pathlib import Path
import csv

@pytest.fixture(scope='session')
def dataframe():
    adresser = [
        'Dronning Dagmars Vej 200, 3650 Ølstykke',
        'Rødkildevej 46, 2400 København',
    ]
    df = pd.DataFrame(data=adresser, columns=['Adresse'])
    return df

@pytest.fixture(scope='session')
def csv_file(dataframe, tmpdir_factory):
    csv_filename = str(tmpdir_factory.mktemp('data').join('test_data.csv'))
    dataframe.to_csv(csv_filename, sep=';')
    return csv_filename 

@pytest.fixture(scope='session')
def xlsx_file(dataframe, tmpdir_factory):
    xlsx_filename = str(tmpdir_factory.mktemp('data').join('test_data.xlsx'))
    dataframe.to_excel(xlsx_filename)
    return xlsx_filename 

@pytest.fixture(scope='session')
def geokoder_object_csv(csv_file):
    geokoder = Geokoder(csv_file, cols=['Adresse'])
    return geokoder

@pytest.fixture(scope='session')
def geokoder_object_xlsx(xlsx_file):
    geokoder = Geokoder(xlsx_file, cols=['Adresse'])
    return geokoder

def test_file_not_found():
    with pytest.raises(FileNotFoundError):
        geokoder = Geokoder('c:/test.csv', cols=['Adresse'])

def test_geokod_csv_save_false(geokoder_object_csv):
    df_geokod = geokoder_object_csv.geokod()
    assert isinstance(df_geokod, pd.DataFrame)
    assert 'x' in df_geokod.columns
    assert 'y' in df_geokod.columns

def test_geokod_csv_save_file(geokoder_object_csv):
    geokoder_object_csv.geokod(save=True)
    saved_file = str(geokoder_object_csv.filepath.parent.joinpath(f'{geokoder_object_csv.filepath.stem}_geokodet{geokoder_object_csv.filepath.suffix}'))
    assert Path(saved_file).exists

def test_geokod_xlsx_save_false(geokoder_object_xlsx):
    df_geokod = geokoder_object_xlsx.geokod()
    assert isinstance(df_geokod, pd.DataFrame)
    assert 'x' in df_geokod.columns
    assert 'y' in df_geokod.columns

def test_geokod_xlsx_save_file(geokoder_object_xlsx):
    geokoder_object_xlsx.geokod(save=True)
    saved_file = str(geokoder_object_xlsx.filepath.parent.joinpath(f'{geokoder_object_xlsx.filepath.stem}_geokodet{geokoder_object_xlsx.filepath.suffix}'))
    assert Path(saved_file).exists

def test_csv_find_sep(geokoder_object_csv):
    sep = geokoder_object_csv.find_csv_sep()
    assert sep == ';'


from setuptools import setup, find_packages

setup(
    name='pydawa',
    version='0.4.0',
    packages=find_packages(),
    license='MIT',
    description='API wrapper til DAWA',
    long_description=open('README.md').read(),
    install_requires=['requests'],
    url='http://github.com/danielarnason/pydawa',
    author='Daníel Örn Árnason',
    author_email='danielarnason85@gmail.com'
)


import pytest
from pydawa import Adressesoeg, Adresseopslag, Adressevasker, Reverse

def test_adressesoeg_opslit():
    adresse = Adressesoeg(vejnavn='Rødkildevej', husnr='46', postnr='2400')
    adresse_info = adresse.info()
    assert isinstance(adresse_info, list)
    assert adresse_info[0]['vejnavn'] == 'Rødkildevej'
    assert adresse_info[0]['husnr'] == '46'
    assert adresse_info[0]['postnr'] == '2400'

def test_adressesoeg_q():
    adresse = Adressesoeg(q='Rødkildevej 46, 2400 København')
    adresse_info = adresse.info()
    assert isinstance(adresse_info, list)
    assert adresse_info[0]['vejnavn'] == 'Rødkildevej'
    assert adresse_info[0]['husnr'] == '46'
    assert adresse_info[0]['postnr'] == '2400'

def test_adresseopslag():
    adresse = Adresseopslag(id='0a3f507a-b2e6-32b8-e044-0003ba298018')
    adresse_info = adresse.info()
    assert isinstance(adresse_info, dict)
    assert adresse_info['vejnavn'] == 'Landgreven'
    assert adresse_info['husnr'] == '10'
    assert adresse_info['postnr'] == '1301'

def test_adressevasker():
    adresse = Adressevasker(betegnelse='Dronning dagmarsvej 200 3650 ølstyke')
    adresse_info = adresse.info()
    assert isinstance(adresse_info, dict)
    assert 'kategori' in adresse_info
    assert isinstance(adresse_info['resultater'], list)

def test_get_koordinater():
    adresse = Adressesoeg(q='Rødkildevej 46, 2400 København')
    adresse_info = adresse.info()
    koordinater = adresse.get_koordinater(adresse_info[0])
    assert isinstance(koordinater, tuple)
    assert isinstance(koordinater[0], float)
    assert isinstance(koordinater[1], float)

def test_reverse():
    koordinater = (12.58, 55.68)
    reverse_geokod = Reverse(koordinater=koordinater).info()
    assert isinstance(reverse_geokod, dict)
    assert reverse_geokod['id'] == '0a3f507a-f1ad-32b8-e044-0003ba298018'